import cv2
import numpy as np
import dlib
from imutils import face_utils
from keras.models import load_model
from statistics import mode
from utils.datasets import get_labels
from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
import threading

# Constants
USE_WEBCAM = True
emotion_model_path = 'models/emotion_model.hdf5'
emotion_labels = get_labels('fer2013')
frame_window = 10
emotion_offsets = (20, 40)

# Load models
detector = dlib.get_frontal_face_detector()
emotion_classifier = load_model(emotion_model_path)
emotion_target_size = emotion_classifier.input_shape[1:3]
emotion_window = []

# Video capture
cap = cv2.VideoCapture(0)

# Shared variable for emotions
emotion_data = []

def detect_emotions():
    global emotion_data
    while cap.isOpened():
        ret, bgr_image = cap.read()
        gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
        rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

        faces = detector(rgb_image)
        for face_coordinates in faces:
            x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
            gray_face = gray_image[y1:y2, x1:x2]
            try:
                gray_face = cv2.resize(gray_face, (emotion_target_size))
            except:
                continue

            gray_face = preprocess_input(gray_face, True)
            gray_face = np.expand_dims(gray_face, 0)
            gray_face = np.expand_dims(gray_face, -1)
            emotion_prediction = emotion_classifier.predict(gray_face)
            emotion_probability = np.max(emotion_prediction)
            emotion_label_arg = np.argmax(emotion_prediction)
            emotion_text = emotion_labels[emotion_label_arg]
            emotion_window.append(emotion_text)

            if len(emotion_window) > frame_window:
                emotion_window.pop(0)
            try:
                emotion_mode = mode(emotion_window)
            except:
                continue

            if emotion_text == 'angry':
                color = emotion_probability * np.asarray((255, 0, 0))
            elif emotion_text == 'sad':
                color = emotion_probability * np.asarray((0, 0, 255))
            elif emotion_text == 'happy':
                color = emotion_probability * np.asarray((255, 255, 0))
            elif emotion_text == 'surprise':
                color = emotion_probability * np.asarray((0, 255, 255))
            else:
                color = emotion_probability * np.asarray((0, 255, 0))

            color = color.astype(int).tolist()

            draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
            draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode, color, 0, -45, 1, 1)

            # Update shared emotion data
            emotion_data = emotion_text

        bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
        cv2.imshow('window_frame', bgr_image)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()

# Start emotion detection in a separate thread
emotion_thread = threading.Thread(target=detect_emotions)
emotion_thread.start()


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# # starting video streaming
# cv2.namedWindow('window_frame')
# video_capture = cv2.VideoCapture(0)

# # Select video or webcam feed
# cap = cv2.VideoCapture(0 if USE_WEBCAM else './test/testvdo.mp4')

# while cap.isOpened():  # True:
#     ret, bgr_image = cap.read()

#     if not ret:
#         print("Failed to grab frame")
#         break

#     gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#     rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#     faces = detector(rgb_image)

#     for face_coordinates in faces:
#         x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#         gray_face = gray_image[y1:y2, x1:x2]
#         try:
#             gray_face = cv2.resize(gray_face, (emotion_target_size))
#         except Exception as e:
#             print(f"Resize failed: {e}")
#             continue

#         gray_face = preprocess_input(gray_face, True)
#         gray_face = np.expand_dims(gray_face, 0)
#         gray_face = np.expand_dims(gray_face, -1)
#         emotion_prediction = emotion_classifier.predict(gray_face)
#         emotion_probability = np.max(emotion_prediction)
#         emotion_label_arg = np.argmax(emotion_prediction)
#         emotion_text = emotion_labels[emotion_label_arg]
#         emotion_window.append(emotion_text)

#         if len(emotion_window) > frame_window:
#             emotion_window.pop(0)
#         try:
#             emotion_mode = mode(emotion_window)
#         except Exception as e:
#             print(f"Mode calculation failed: {e}")
#             continue

#         if emotion_text == 'angry':
#             color = emotion_probability * np.asarray((255, 0, 0))
#         elif emotion_text == 'sad':
#             color = emotion_probability * np.asarray((0, 0, 255))
#         elif emotion_text == 'happy':
#             color = emotion_probability * np.asarray((255, 255, 0))
#         elif emotion_text == 'surprise':
#             color = emotion_probability * np.asarray((0, 255, 255))
#         else:
#             color = emotion_probability * np.asarray((0, 255, 0))

#         color = color.astype(int)
#         color = color.tolist()

#         draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#         draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode,
#                   color, 0, -45, 1, 1)

#     bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#     cv2.imshow('window_frame', bgr_image)
#     if cv2.waitKey(1) & 0xFF == ord('q'):
#         break

# cap.release()
# cv2.destroyAllWindows()

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import requests

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# # starting video streaming
# cv2.namedWindow('window_frame')
# video_capture = cv2.VideoCapture(0)

# # Flask server URL
# FLASK_URL = 'http://localhost:5000/update_emotion'

# # Select video or webcam feed
# cap = cv2.VideoCapture(0)

# while cap.isOpened():  # True:
#     ret, bgr_image = cap.read()

#     if not ret:
#         print("Failed to grab frame")
#         break

#     gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#     rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#     faces = detector(rgb_image)

#     for face_coordinates in faces:
#         x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#         gray_face = gray_image[y1:y2, x1:x2]
#         try:
#             gray_face = cv2.resize(gray_face, (emotion_target_size))
#         except Exception as e:
#             print(f"Resize failed: {e}")
#             continue

#         gray_face = preprocess_input(gray_face, True)
#         gray_face = np.expand_dims(gray_face, 0)
#         gray_face = np.expand_dims(gray_face, -1)
#         emotion_prediction = emotion_classifier.predict(gray_face)
#         emotion_probability = np.max(emotion_prediction)
#         emotion_label_arg = np.argmax(emotion_prediction)
#         emotion_text = emotion_labels[emotion_label_arg]
#         emotion_window.append(emotion_text)

#         if len(emotion_window) > frame_window:
#             emotion_window.pop(0)
#         try:
#             emotion_mode = mode(emotion_window)
#         except Exception as e:
#             print(f"Mode calculation failed: {e}")
#             continue

#         if emotion_text == 'angry':
#             color = emotion_probability * np.asarray((255, 0, 0))
#         elif emotion_text == 'sad':
#             color = emotion_probability * np.asarray((0, 0, 255))
#         elif emotion_text == 'happy':
#             color = emotion_probability * np.asarray((255, 255, 0))
#         elif emotion_text == 'surprise':
#             color = emotion_probability * np.asarray((0, 255, 255))
#         else:
#             color = emotion_probability * np.asarray((0, 255, 0))

#         color = color.astype(int)
#         color = color.tolist()

#         draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#         draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode,
#                   color, 0, -45, 1, 1)

#         # Send emotion data to Flask server
#         try:
#             response = requests.post(FLASK_URL, json={'emotion': emotion_mode})
#             print(f"Sent emotion data: {response.status_code}")
#         except Exception as e:
#             print(f"Failed to send emotion data: {e}")

#     bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#     cv2.imshow('window_frame', bgr_image)
#     if cv2.waitKey(1) & 0xFF == ord('q'):
#         break

# cap.release()
# cv2.destroyAllWindows()

# ADDING DEBUGGING TO SEE IF ITS SENDING THE CORRECT INFO

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import requests
# import logging

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# # starting video streaming
# cv2.namedWindow('window_frame')
# video_capture = cv2.VideoCapture(0)

# # Flask server URL
# FLASK_URL = 'http://172.16.20.180:5000/update_emotion'

# # Select video or webcam feed
# cap = cv2.VideoCapture(0)

# while cap.isOpened():  # True:
#     ret, bgr_image = cap.read()

#     if not ret:
#         logger.error("Failed to grab frame")
#         break

#     gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#     rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#     faces = detector(rgb_image)

#     for face_coordinates in faces:
#         x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#         gray_face = gray_image[y1:y2, x1:x2]
#         try:
#             gray_face = cv2.resize(gray_face, (emotion_target_size))
#         except Exception as e:
#             logger.error(f"Resize failed: {e}")
#             continue

#         gray_face = preprocess_input(gray_face, True)
#         gray_face = np.expand_dims(gray_face, 0)
#         gray_face = np.expand_dims(gray_face, -1)
#         emotion_prediction = emotion_classifier.predict(gray_face)
#         emotion_probability = np.max(emotion_prediction)
#         emotion_label_arg = np.argmax(emotion_prediction)
#         emotion_text = emotion_labels[emotion_label_arg]
#         emotion_window.append(emotion_text)

#         if len(emotion_window) > frame_window:
#             emotion_window.pop(0)
#         try:
#             emotion_mode = mode(emotion_window)
#         except Exception as e:
#             logger.error(f"Mode calculation failed: {e}")
#             continue

#         logger.debug(f"Detected emotion: {emotion_mode}")

#         if emotion_text == 'angry':
#             color = emotion_probability * np.asarray((255, 0, 0))
#         elif emotion_text == 'sad':
#             color = emotion_probability * np.asarray((0, 0, 255))
#         elif emotion_text == 'happy':
#             color = emotion_probability * np.asarray((255, 255, 0))
#         elif emotion_text == 'surprise':
#             color = emotion_probability * np.asarray((0, 255, 255))
#         else:
#             color = emotion_probability * np.asarray((0, 255, 0))

#         color = color.astype(int)
#         color = color.tolist()

#         draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#         draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode,
#                   color, 0, -45, 1, 1)

#         # Send emotion data to Flask server
#         try:
#             response = requests.post(FLASK_URL, json={'emotion': emotion_mode})
#             logger.debug(f"Sent emotion data: {response.status_code}")
#         except Exception as e:
#             logger.error(f"Failed to send emotion data: {e}")

#     bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#     cv2.imshow('window_frame', bgr_image)
#     if cv2.waitKey(1) & 0xFF == ord('q'):
#         break

# cap.release()
# cv2.destroyAllWindows()

# TESTING CODE THAT ASKS A QUESTION

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import apply_offsets, preprocess_input
# import requests
# import logging
# from flask import Flask, render_template, request

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# # WiFi server details
# ESP32_IP = 'http://172.16.22.121'  # Replace with your ESP32 IP address
# SENSOR_DATA_URL = f'{ESP32_IP}/sensor-data'

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# def fetch_sensor_data():
#     try:
#         response = requests.get(SENSOR_DATA_URL)
#         response.raise_for_status()
#         return response.json()
#     except requests.RequestException as e:
#         logger.error(f"Error fetching sensor data: {e}")
#         return None

# def detect_emotion():
#     while True:
#         sensor_data = fetch_sensor_data()
#         if sensor_data:
#             # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#             total_pressure = sum(sensor_data.values())
#             if total_pressure > 2500:  # Adjust threshold as needed
#                 cap = cv2.VideoCapture(0)
#                 while cap.isOpened():  # True:
#                     ret, bgr_image = cap.read()
#                     if not ret:
#                         logger.error("Failed to grab frame")
#                         break

#                     gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#                     rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#                     faces = detector(rgb_image)
#                     for face_coordinates in faces:
#                         x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                         gray_face = gray_image[y1:y2, x1:x2]
#                         try:
#                             gray_face = cv2.resize(gray_face, (emotion_target_size))
#                         except Exception as e:
#                             logger.error(f"Resize failed: {e}")
#                             continue

#                         gray_face = preprocess_input(gray_face, True)
#                         gray_face = np.expand_dims(gray_face, 0)
#                         gray_face = np.expand_dims(gray_face, -1)
#                         emotion_prediction = emotion_classifier.predict(gray_face)
#                         emotion_probability = np.max(emotion_prediction)
#                         emotion_label_arg = np.argmax(emotion_prediction)
#                         emotion_text = emotion_labels[emotion_label_arg]
#                         emotion_window.append(emotion_text)

#                         if len(emotion_window) > frame_window:
#                             emotion_window.pop(0)
#                         try:
#                             emotion_mode = mode(emotion_window)
#                         except Exception as e:
#                             logger.error(f"Mode calculation failed: {e}")
#                             continue

#                         logger.debug(f"Detected emotion: {emotion_mode}")

#                         # Redirect to Flask app with detected emotion
#                         return emotion_mode

#                 cap.release()
#                 cv2.destroyAllWindows()
#         time.sleep(2)  # Adjust the polling interval as needed

# @app.route('/')
# def index():
#     detected_emotion = detect_emotion()
#     return render_template('index.html', emotion=detected_emotion)

# @app.route('/question', methods=['POST'])
# def question():
#     emotion = request.form['emotion']
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# if __name__ == '__main__':
#     app.run(debug=True)


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import apply_offsets, preprocess_input
# import requests
# import logging
# from flask import Flask, render_template, request

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# # WiFi server details
# ESP32_IP = 'http://172.16.22.121'  # Replace with your ESP32 IP address
# SENSOR_DATA_URL = f'{ESP32_IP}/sensor-data'

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# def fetch_sensor_data():
#     try:
#         response = requests.get(SENSOR_DATA_URL)
#         response.raise_for_status()
#         return response.json()
#     except requests.RequestException as e:
#         logger.error(f"Error fetching sensor data: {e}")
#         return None

# def detect_emotion():
#     sensor_data = fetch_sensor_data()
#     if sensor_data:
#         # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#         total_pressure = sum(sensor_data.values())
#         if total_pressure > 2500:  # Adjust threshold as needed
#             cap = cv2.VideoCapture(0)
#             while cap.isOpened():  # True:
#                 ret, bgr_image = cap.read()
#                 if not ret:
#                     logger.error("Failed to grab frame")
#                     break

#                 gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#                 rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#                 faces = detector(rgb_image)
#                 for face_coordinates in faces:
#                     x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                     gray_face = gray_image[y1:y2, x1:x2]
#                     try:
#                         gray_face = cv2.resize(gray_face, (emotion_target_size))
#                     except Exception as e:
#                         logger.error(f"Resize failed: {e}")
#                         continue

#                     gray_face = preprocess_input(gray_face, True)
#                     gray_face = np.expand_dims(gray_face, 0)
#                     gray_face = np.expand_dims(gray_face, -1)
#                     emotion_prediction = emotion_classifier.predict(gray_face)
#                     emotion_probability = np.max(emotion_prediction)
#                     emotion_label_arg = np.argmax(emotion_prediction)
#                     emotion_text = emotion_labels[emotion_label_arg]
#                     emotion_window.append(emotion_text)

#                     if len(emotion_window) > frame_window:
#                         emotion_window.pop(0)
#                     try:
#                         emotion_mode = mode(emotion_window)
#                     except Exception as e:
#                         logger.error(f"Mode calculation failed: {e}")
#                         continue

#                     logger.debug(f"Detected emotion: {emotion_mode}")

#                     # Redirect to Flask app with detected emotion
#                     return emotion_mode

#             cap.release()
#             cv2.destroyAllWindows()
#     time.sleep(2)  # Adjust the polling interval as needed

# @app.route('/')
# def index():
#     detected_emotion = detect_emotion()
#     if detected_emotion:
#         question = get_question(detected_emotion)
#         return render_template('index.html', question=question)
#     else:
#         return "No emotion detected at the moment."

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# if __name__ == '__main__':
#     app.run(debug=True)

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import apply_offsets, preprocess_input
# import requests
# import logging
# from flask import Flask, render_template, request, Response
# import time

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# # WiFi server details
# ESP32_IP = 'http://172.16.22.121'  # Replace with your ESP32 IP address
# SENSOR_DATA_URL = f'{ESP32_IP}/sensor-data'

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# def fetch_sensor_data():
#     try:
#         response = requests.get(SENSOR_DATA_URL)
#         response.raise_for_status()
#         return response.json()
#     except requests.RequestException as e:
#         logger.error(f"Error fetching sensor data: {e}")
#         return None

# def detect_emotion():
#     sensor_data = fetch_sensor_data()
#     if sensor_data:
#         # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#         total_pressure = sum(sensor_data.values())
#         if total_pressure > 2500:  # Adjust threshold as needed
#             cap = cv2.VideoCapture(0)
#             while cap.isOpened():  # True:
#                 ret, bgr_image = cap.read()
#                 if not ret:
#                     logger.error("Failed to grab frame")
#                     break

#                 gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#                 rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#                 faces = detector(rgb_image)
#                 for face_coordinates in faces:
#                     x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                     gray_face = gray_image[y1:y2, x1:x2]
#                     try:
#                         gray_face = cv2.resize(gray_face, (emotion_target_size))
#                     except Exception as e:
#                         logger.error(f"Resize failed: {e}")
#                         continue

#                     gray_face = preprocess_input(gray_face, True)
#                     gray_face = np.expand_dims(gray_face, 0)
#                     gray_face = np.expand_dims(gray_face, -1)
#                     emotion_prediction = emotion_classifier.predict(gray_face)
#                     emotion_probability = np.max(emotion_prediction)
#                     emotion_label_arg = np.argmax(emotion_prediction)
#                     emotion_text = emotion_labels[emotion_label_arg]
#                     emotion_window.append(emotion_text)

#                     if len(emotion_window) > frame_window:
#                         emotion_window.pop(0)
#                     try:
#                         emotion_mode = mode(emotion_window)
#                     except Exception as e:
#                         logger.error(f"Mode calculation failed: {e}")
#                         continue

#                     logger.debug(f"Detected emotion: {emotion_mode}")

#                     yield (b'--frame\r\n'
#                            b'Content-Type: image/jpeg\r\n\r\n' + cv2.imencode('.jpg', bgr_image)[1].tobytes() + b'\r\n')

#             cap.release()
#             cv2.destroyAllWindows()
#     time.sleep(2)  # Adjust the polling interval as needed

# @app.route('/')
# def index():
#     return render_template('index.html')

# @app.route('/video_feed')
# def video_feed():
#     return Response(detect_emotion(), mimetype='multipart/x-mixed-replace; boundary=frame')

# if __name__ == '__main__':
#     app.run(debug=True)


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# @app.route('/')
# def index():
#     return render_template('index.html')

# def gen_frames():  # generate frame by frame from camera
#     while True:
#         cap = cv2.VideoCapture(0)
#         success, frame = cap.read()  # read the camera frame
#         if not success:
#             break
#         else:
#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')  # concat frame one by one and show result

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# def fetch_sensor_data():
#     # Code to fetch sensor data from ESP32
#     pass

# def detect_emotion():
#     while True:
#         sensor_data = fetch_sensor_data()
#         if sensor_data:
#             # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#             total_pressure = sum(sensor_data.values())
#             if total_pressure > 2500:  # Adjust threshold as needed
#                 cap = cv2.VideoCapture(0)
#                 while cap.isOpened():  # True:
#                     ret, bgr_image = cap.read()
#                     if not ret:
#                         logger.error("Failed to grab frame")
#                         break

#                     gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#                     rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#                     faces = detector(rgb_image)
#                     for face_coordinates in faces:
#                         x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                         gray_face = gray_image[y1:y2, x1:x2]
#                         try:
#                             gray_face = cv2.resize(gray_face, (emotion_target_size))
#                         except Exception as e:
#                             logger.error(f"Resize failed: {e}")
#                             continue

#                         gray_face = preprocess_input(gray_face, True)
#                         gray_face = np.expand_dims(gray_face, 0)
#                         gray_face = np.expand_dims(gray_face, -1)
#                         emotion_prediction = emotion_classifier.predict(gray_face)
#                         emotion_probability = np.max(emotion_prediction)
#                         emotion_label_arg = np.argmax(emotion_prediction)
#                         emotion_text = emotion_labels[emotion_label_arg]
#                         emotion_window.append(emotion_text)

#                         if len(emotion_window) > frame_window:
#                             emotion_window.pop(0)
#                         try:
#                             emotion_mode = mode(emotion_window)
#                         except Exception as e:
#                             logger.error(f"Mode calculation failed: {e}")
#                             continue

#                         logger.debug(f"Detected emotion: {emotion_mode}")

#                         # Redirect to Flask app with detected emotion
#                         return show_emotion(emotion_mode)

#                 cap.release()
#                 cv2.destroyAllWindows()
#         time.sleep(2)  # Adjust the polling interval as needed

# if __name__ == '__main__':
#     app.run(debug=True)

#  WORKS TO SHOW THE VIDEO WITH THE TEXT ABOVE IT BUT HAS A BLUE TINT AND ONLY SHOWS THE ELSE
# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response
# import time

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down."
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath."
#     else:
#         return "It seems you're feeling a bit off."

# def gen_frames():  # generate frame by frame from camera
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()  # read the camera frame
#         if not success:
#             break
#         else:
#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')  # concat frame one by one and show result

# @app.route('/')
# def index():
#     emotion_mode = detect_emotion()  # Assuming detect_emotion returns the detected emotion
#     question = get_question(emotion_mode)  # Assuming get_question returns the question based on the detected emotion
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# def fetch_sensor_data():
#     # Code to fetch sensor data from ESP32
#     pass

# def detect_emotion():
#     sensor_data = fetch_sensor_data()
#     if sensor_data:
#         # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#         total_pressure = sum(sensor_data.values())
#         if total_pressure > 2500:  # Adjust threshold as needed
#             cap = cv2.VideoCapture(0)
#             while cap.isOpened():  # True:
#                 ret, bgr_image = cap.read()
#                 if not ret:
#                     logger.error("Failed to grab frame")
#                     break

#                 gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#                 rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#                 faces = detector(rgb_image)
#                 for face_coordinates in faces:
#                     x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                     gray_face = gray_image[y1:y2, x1:x2]
#                     try:
#                         gray_face = cv2.resize(gray_face, (emotion_target_size))
#                     except Exception as e:
#                         logger.error(f"Resize failed: {e}")
#                         continue

#                     gray_face = preprocess_input(gray_face, True)
#                     gray_face = np.expand_dims(gray_face, 0)
#                     gray_face = np.expand_dims(gray_face, -1)
#                     emotion_prediction = emotion_classifier.predict(gray_face)
#                     emotion_probability = np.max(emotion_prediction)
#                     emotion_label_arg = np.argmax(emotion_prediction)
#                     emotion_text = emotion_labels[emotion_label_arg]
#                     emotion_window.append(emotion_text)

#                     if len(emotion_window) > frame_window:
#                         emotion_window.pop(0)
#                     try:
#                         emotion_mode = mode(emotion_window)
#                     except Exception as e:
#                         logger.error(f"Mode calculation failed: {e}")
#                         continue

#                     logger.debug(f"Detected emotion: {emotion_mode}")

#                     return emotion_mode  # Return detected emotion

#             cap.release()
#             cv2.destroyAllWindows()
#     time.sleep(2)  # Adjust the polling interval as needed
#     return "neutral"  # Return neutral emotion if no sensor data or threshold condition not met

# if __name__ == '__main__':
#     app.run(debug=True)

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response
# import time

# app = Flask(__name__)

# # Set up logging configuration
# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True  # If false, loads video file source

# # parameters for loading data and images
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')

# # hyper-parameters for bounding boxes shape
# frame_window = 10
# emotion_offsets = (20, 40)

# # loading models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)

# # getting input model shapes for inference
# emotion_target_size = emotion_classifier.input_shape[1:3]

# # starting lists for calculating modes
# emotion_window = []

# # Placeholder for sensor data
# sensor_data = {'pressure': 0}  # Update with actual sensor data received from ESP32

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def gen_frames():  # generate frame by frame from camera
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()  # read the camera frame
#         if not success:
#             break
#         else:
#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')  # concat frame one by one and show result

# @app.route('/')
# def index():
#     emotion_mode = detect_emotion()  # Assuming detect_emotion returns the detected emotion
#     question = get_question(emotion_mode)  # Assuming get_question returns the question based on the detected emotion
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# def fetch_sensor_data():
#     # Placeholder for fetching sensor data from ESP32
#     global sensor_data
#     # Example: Update sensor_data based on actual sensor data received from ESP32
#     sensor_data = {'pressure': 3000}  # Update with actual sensor data received from ESP32

# def detect_emotion():
#     fetch_sensor_data()
#     # Example: If the sum of sensor values is greater than a threshold, activate facial recognition
#     total_pressure = sensor_data.get('pressure', 0)
#     if total_pressure > 2500:  # Adjust threshold as needed
#         cap = cv2.VideoCapture(0)
#         while cap.isOpened():  # True:
#             ret, bgr_image = cap.read()
#             if not ret:
#                 logger.error("Failed to grab frame")
#                 break

#             gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#             rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#             faces = detector(rgb_image)
#             for face_coordinates in faces:
#                 x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                 gray_face = gray_image[y1:y2, x1:x2]
#                 try:
#                     gray_face = cv2.resize(gray_face, (emotion_target_size))
#                 except Exception as e:
#                     logger.error(f"Resize failed: {e}")
#                     continue

#                 gray_face = preprocess_input(gray_face, True)
#                 gray_face = np.expand_dims(gray_face, 0)
#                 gray_face = np.expand_dims(gray_face, -1)
#                 emotion_prediction = emotion_classifier.predict(gray_face)
#                 emotion_probability = np.max(emotion_prediction)
#                 emotion_label_arg = np.argmax(emotion_prediction)
#                 emotion_text = emotion_labels[emotion_label_arg]
#                 emotion_window.append(emotion_text)

#                 if len(emotion_window) > frame_window:
#                     emotion_window.pop(0)
#                 try:
#                     emotion_mode = mode(emotion_window)
#                 except Exception as e:
#                     logger.error(f"Mode calculation failed: {e}")
#                     continue

#                 logger.debug(f"Detected emotion: {emotion_mode}")

#                 return emotion_mode  # Return detected emotion

#         cap.release()
#         cv2.destroyAllWindows()

#     return "neutral"  # Return neutral emotion if sensor data does not meet threshold

# if __name__ == '__main__':
#     app.run(debug=True)

# ONLY ACTIVATING IF THE PRESSURE IS ABOVE 3000
# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response, request
# import time
# import threading
# from werkzeug.serving import run_simple
# import signal
# import sys

# app = Flask(__name__)

# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []
# sensor_data = {'pressure': 18000}

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def gen_frames():
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()
#         if not success:
#             break
#         else:
#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')

# @app.route('/')
# def index():
#     emotion_mode = detect_emotion()
#     question = get_question(emotion_mode)
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# @app.route('/trigger-python', methods=['POST'])
# def trigger_python():
#     emotion_mode = detect_emotion()
#     logger.info(f"Triggered from ESP32. Detected emotion: {emotion_mode}")
#     return "Python code triggered!", 200

# def fetch_sensor_data():
#     global sensor_data
#     incomingValues = [1000, 2000, 3000, 1500, 2500, 3500, 1200, 2200, 3200]
#     total_pressure = sum(incomingValues)
#     sensor_data = {'pressure': total_pressure}

# def detect_emotion():
#     fetch_sensor_data()
#     total_pressure = sensor_data.get('pressure', 0)
#     if total_pressure > 28655:
#         cap = cv2.VideoCapture(0)
#         while cap.isOpened():
#             ret, bgr_image = cap.read()
#             if not ret:
#                 logger.error("Failed to grab frame")
#                 break

#             gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#             rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#             faces = detector(rgb_image)
#             for face_coordinates in faces:
#                 x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                 gray_face = gray_image[y1:y2, x1:x2]
#                 try:
#                     gray_face = cv2.resize(gray_face, (emotion_target_size))
#                 except Exception as e:
#                     logger.error(f"Resize failed: {e}")
#                     continue

#                 gray_face = preprocess_input(gray_face, True)
#                 gray_face = np.expand_dims(gray_face, 0)
#                 gray_face = np.expand_dims(gray_face, -1)
#                 emotion_prediction = emotion_classifier.predict(gray_face)
#                 emotion_probability = np.max(emotion_prediction)
#                 emotion_label_arg = np.argmax(emotion_prediction)
#                 emotion_text = emotion_labels[emotion_label_arg]
#                 emotion_window.append(emotion_text)

#                 if len(emotion_window) > frame_window:
#                     emotion_window.pop(0)
#                 try:
#                     emotion_mode = mode(emotion_window)
#                 except Exception as e:
#                     logger.error(f"Mode calculation failed: {e}")
#                     continue

#                 logger.debug(f"Detected emotion: {emotion_mode}")

#                 return emotion_mode

#         cap.release()
#         cv2.destroyAllWindows()

#     return "neutral"

# def reload_server():
#     while True:
#         time.sleep(5)  # Wait for 5 seconds
#         app.logger.info('Reloading Flask server...')
#         app.logger.info('Flask server reloaded successfully.')
#         run_simple('localhost', 5000, app)

# if __name__ == '__main__':
#     sensor_thread = threading.Thread(target=detect_emotion)
#     sensor_thread.daemon = True
#     sensor_thread.start()
    
#     reload_thread = threading.Thread(target=reload_server)
#     reload_thread.daemon = True
#     reload_thread.start()
    
#     app.run(debug=True)

# CORRECTLY FETCHING DATA EVERY 5 SECONDS BUT NOT UPDATING THE TEXT

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response, request
# import threading
# import time

# app = Flask(__name__)

# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []
# sensor_data = {'pressure': 18000}

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def gen_frames():
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()
#         if not success:
#             break
#         else:
#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')

# @app.route('/')
# def index():
#     emotion_mode = detect_emotion()
#     question = get_question(emotion_mode)
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# @app.route('/trigger-python', methods=['POST'])
# def trigger_python():
#     logger.debug('Received trigger from ESP32.')
#     emotion_mode = detect_emotion()
#     logger.info(f"Triggered from ESP32. Detected emotion: {emotion_mode}")
#     return "Python code triggered!", 200

# def fetch_sensor_data():
#     global sensor_data
#     incomingValues = [1000, 2000, 3000, 1500, 2500, 3500, 1200, 2200, 3200]
#     total_pressure = sum(incomingValues)
#     sensor_data = {'pressure': total_pressure}
#     logger.debug(f"Fetched sensor data: {sensor_data}")

# def check_sensor_data():
#     while True:
#         fetch_sensor_data()
#         total_pressure = sensor_data.get('pressure', 0)
#         logger.debug(f"Total pressure: {total_pressure}")
#         if total_pressure > 28655:
#             detect_emotion()
#         time.sleep(5)  # Check every 5 seconds

# def detect_emotion():
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             logger.error("Failed to grab frame")
#             break

#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except Exception as e:
#                 logger.error(f"Resize failed: {e}")
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except Exception as e:
#                 logger.error(f"Mode calculation failed: {e}")
#                 continue

#             logger.debug(f"Detected emotion: {emotion_mode}")

#             return emotion_mode

#     cap.release()
#     cv2.destroyAllWindows()
#     return "neutral"

# if __name__ == '__main__':
#     sensor_thread = threading.Thread(target=check_sensor_data)
#     sensor_thread.daemon = True
#     sensor_thread.start()
    
#     app.run(debug=True, host='0.0.0.0', port=5000)


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response, request, jsonify
# import threading
# import time

# app = Flask(__name__)

# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []
# sensor_data = {'pressure': 18000}
# current_emotion = "neutral"

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def detect_emotions():
#     global current_emotion
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             logger.error("Failed to grab frame")
#             break

#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except Exception as e:
#                 logger.error(f"Resize failed: {e}")
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 current_emotion = mode(emotion_window)
#             except Exception as e:
#                 logger.error(f"Mode calculation failed: {e}")
#                 continue

#             logger.debug(f"Detected emotion: {current_emotion}")

#             draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, (0, 255, 0))
#             draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, current_emotion, (0, 255, 0), 0, -45, 1, 1)

#     cap.release()
#     cv2.destroyAllWindows()

# # Start emotion detection in a separate thread
# emotion_thread = threading.Thread(target=detect_emotions)
# emotion_thread.daemon = True
# emotion_thread.start()

# @app.route('/')
# def index():
#     global current_emotion
#     question = get_question(current_emotion)
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/emotion/<emotion_mode>')
# def show_emotion(emotion_mode):
#     return render_template('emotion.html', emotion=emotion_mode)

# @app.route('/trigger-python', methods=['POST'])
# def trigger_python():
#     global current_emotion
#     logger.debug('Received trigger from ESP32.')
#     current_emotion = detect_emotion()
#     logger.info(f"Triggered from ESP32. Detected emotion: {current_emotion}")
#     return "Python code triggered!", 200

# @app.route('/update_emotion')
# def update_emotion():
#     global current_emotion
#     question = get_question(current_emotion)
#     return jsonify(question=question)

# if __name__ == '__main__':
#     app.run(debug=True, host='0.0.0.0', port=5000)

# VIDEO CORRECT COLORING AND SHOWING THE EMOTION BY THE FACE BUT NOT CHANGING SENTENCE

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response, request, jsonify
# import threading
# import time

# app = Flask(__name__)

# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []
# sensor_data = {'pressure': 18000}
# current_emotion = "neutral"

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def gen_frames():
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()
#         if not success:
#             break
#         else:
#             gray_image = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
#             rgb_image = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

#             # Detect faces and emotions
#             faces = detector(rgb_image)
#             for face_coordinates in faces:
#                 x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                 gray_face = gray_image[y1:y2, x1:x2]
#                 try:
#                     gray_face = cv2.resize(gray_face, (emotion_target_size))
#                 except Exception as e:
#                     logger.error(f"Resize failed: {e}")
#                     continue

#                 gray_face = preprocess_input(gray_face, True)
#                 gray_face = np.expand_dims(gray_face, 0)
#                 gray_face = np.expand_dims(gray_face, -1)
#                 emotion_prediction = emotion_classifier.predict(gray_face)
#                 emotion_probability = np.max(emotion_prediction)
#                 emotion_label_arg = np.argmax(emotion_prediction)
#                 current_emotion = emotion_labels[emotion_label_arg]
#                 emotion_window.append(current_emotion)

#                 if len(emotion_window) > frame_window:
#                     emotion_window.pop(0)
#                 try:
#                     current_emotion = mode(emotion_window)
#                 except Exception as e:
#                     logger.error(f"Mode calculation failed: {e}")
#                     continue

#                 logger.debug(f"Detected emotion: {current_emotion}")

#                 # Draw bounding box and emotion text
#                 draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, (0, 255, 0))
#                 draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, current_emotion, (0, 255, 0), 0, -45, 1, 1)

#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')

# @app.route('/')
# def index():
#     global current_emotion
#     question = get_question(current_emotion)
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# if __name__ == '__main__':
#     app.run(debug=True, host='0.0.0.0', port=5000)

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import logging
# from flask import Flask, render_template, Response, request, jsonify
# import threading
# import time

# app = Flask(__name__)

# logging.basicConfig(level=logging.DEBUG)
# logger = logging.getLogger(__name__)

# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []
# sensor_data = {'pressure': 18000}
# current_emotion = "neutral"

# def get_question(emotion):
#     if emotion == 'happy':
#         return "How was your day? It looks like you're feeling happy!"
#     elif emotion == 'sad':
#         return "I'm sorry to see you feeling down. Do you want to talk about it?"
#     elif emotion == 'angry':
#         return "Feeling angry? Take a deep breath and let's find a solution together."
#     else:
#         return "It seems you're feeling a bit off. Let's see how we can help."

# def gen_frames():
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         success, frame = cap.read()
#         if not success:
#             break
#         else:
#             gray_image = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
#             rgb_image = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

#             # Detect faces and emotions
#             faces = detector(rgb_image)
#             for face_coordinates in faces:
#                 x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#                 gray_face = gray_image[y1:y2, x1:x2]
#                 try:
#                     gray_face = cv2.resize(gray_face, (emotion_target_size))
#                 except Exception as e:
#                     logger.error(f"Resize failed: {e}")
#                     continue

#                 gray_face = preprocess_input(gray_face, True)
#                 gray_face = np.expand_dims(gray_face, 0)
#                 gray_face = np.expand_dims(gray_face, -1)
#                 emotion_prediction = emotion_classifier.predict(gray_face)
#                 emotion_probability = np.max(emotion_prediction)
#                 emotion_label_arg = np.argmax(emotion_prediction)
#                 current_emotion = emotion_labels[emotion_label_arg]
#                 emotion_window.append(current_emotion)

#                 if len(emotion_window) > frame_window:
#                     emotion_window.pop(0)
#                 try:
#                     current_emotion = mode(emotion_window)
#                 except Exception as e:
#                     logger.error(f"Mode calculation failed: {e}")
#                     continue

#                 logger.debug(f"Detected emotion: {current_emotion}")

#                 # Draw bounding box and emotion text
#                 draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, (0, 255, 0))
#                 draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, current_emotion, (0, 255, 0), 0, -45, 1, 1)

#             ret, buffer = cv2.imencode('.jpg', cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR))
#             frame = buffer.tobytes()
#             yield (b'--frame\r\n'
#                    b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')

# @app.route('/')
# def index():
#     global current_emotion
#     question = get_question(current_emotion)
#     return render_template('index.html', question=question)

# @app.route('/video_feed')
# def video_feed():
#     return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')

# @app.route('/update_emotion')
# def update_emotion():
#     global current_emotion
#     question = get_question(current_emotion)
#     return jsonify(question=question)

# def fetch_sensor_data():
#     global sensor_data
#     incomingValues = [1000, 2000, 3000, 1500, 2500, 3500, 1200, 2200, 3200]
#     total_pressure = sum(incomingValues)
#     sensor_data = {'pressure': total_pressure}
#     logger.debug(f"Fetched sensor data: {sensor_data}")

# def check_sensor_data():
#     while True:
#         fetch_sensor_data()
#         total_pressure = sensor_data.get('pressure', 0)
#         logger.debug(f"Total pressure: {total_pressure}")
#         if total_pressure > 28655:
#                         detect_emotion()
#         time.sleep(5)  # Check every 5 seconds

# def detect_emotion():
#     global current_emotion
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             logger.error("Failed to grab frame")
#             break

#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_RGB2BGR)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except Exception as e:
#                 logger.error(f"Resize failed: {e}")
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             current_emotion = emotion_labels[emotion_label_arg]
#             emotion_window.append(current_emotion)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 current_emotion = mode(emotion_window)
#             except Exception as e:
#                 logger.error(f"Mode calculation failed: {e}")
#                 continue

#             logger.debug(f"Detected emotion: {current_emotion}")

#     cap.release()
#     cv2.destroyAllWindows()
#     return "neutral"

# if __name__ == '__main__':
#     sensor_thread = threading.Thread(target=check_sensor_data)
#     sensor_thread.daemon = True
#     sensor_thread.start()
    
#     app.run(debug=True, host='0.0.0.0', port=5000)

# ----------------------------------------------------------
# ----------------------------------------------------------
# ----------------------------------------------------------
# ----------------------------------------------------------

# SYSTEM OTHER WAY AROUND 

# How the system should work:
# 1. Face is detected
# 2. Emotion is displayed on the face with rectangle
# 3. Text pops up saying "touch me" 
# 4. Pressure matrix detected
# 5. Values of pressure + intensity of the emotion translated to the pumps in intensity
# 6. Depending on the cases a distinct text pops up

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import requests
# import json

# # Constants
# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# ESP32_IP = 'http://192.168.0.17/emotion'

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Video capture
# cap = cv2.VideoCapture(0)

# # Shared variable for emotions
# emotion_data = []

# def detect_emotions():
#     global emotion_data
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             if emotion_text == 'angry':
#                 color = emotion_probability * np.asarray((255, 0, 0))
#             elif emotion_text == 'sad':
#                 color = emotion_probability * np.asarray((0, 0, 255))
#             elif emotion_text == 'happy':
#                 color = emotion_probability * np.asarray((255, 255, 0))
#             elif emotion_text == 'surprise':
#                 color = emotion_probability * np.asarray((0, 255, 255))
#             else:
#                 color = emotion_probability * np.asarray((0, 255, 0))

#             color = color.astype(int).tolist()

#             draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#             draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode, color, 0, -45, 1, 1)

#             # Update shared emotion data
#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#             # Send emotion data to ESP32
#             try:
#                 response = requests.post(ESP32_IP, json=emotion_data)
#                 print(f'Sent data to ESP32: {response.status_code}')
#             except Exception as e:
#                 print(f'Error sending data to ESP32: {e}')

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         cv2.imshow('window_frame', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# # Start emotion detection in the main thread
# if __name__ == "__main__":
#     detect_emotions()

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import requests
# import json

# # Constants
# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# ESP32_IP = 'http://192.168.0.17/emotion'

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Video capture
# cap = cv2.VideoCapture(0)

# # Shared variable for emotions
# emotion_data = []

# def detect_emotions():
#     global emotion_data
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             if emotion_text == 'angry':
#                 color = emotion_probability * np.asarray((255, 0, 0))
#             elif emotion_text == 'sad':
#                 color = emotion_probability * np.asarray((0, 0, 255))
#             elif emotion_text == 'happy':
#                 color = emotion_probability * np.asarray((255, 255, 0))
#             elif emotion_text == 'surprise':
#                 color = emotion_probability * np.asarray((0, 255, 255))
#             else:
#                 color = emotion_probability * np.asarray((0, 255, 0))

#             color = color.astype(int).tolist()

#             draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#             draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode, color, 0, -45, 1, 1)

#             # Update shared emotion data
#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#             # Send emotion data to ESP32
#             try:
#                 response = requests.post(ESP32_IP, json=emotion_data)
#                 print(f'Sent data to ESP32: {response.status_code}')
#             except Exception as e:
#                 print(f'Error sending data to ESP32: {e}')

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         cv2.imshow('window_frame', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# # Start emotion detection in the main thread
# if __name__ == "__main__":
#     detect_emotions()


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import requests
# import json
# from flask import Flask, request, jsonify

# app = Flask(__name__)

# # Constants
# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# ESP32_IP = 'http://192.168.0.17/emotion'

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Video capture
# cap = cv2.VideoCapture(0)

# # Shared variable for emotions
# emotion_data = []

# def detect_emotions():
#     global emotion_data
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             if emotion_text == 'angry':
#                 color = emotion_probability * np.asarray((255, 0, 0))
#             elif emotion_text == 'sad':
#                 color = emotion_probability * np.asarray((0, 0, 255))
#             elif emotion_text == 'happy':
#                 color = emotion_probability * np.asarray((255, 255, 0))
#             elif emotion_text == 'surprise':
#                 color = emotion_probability * np.asarray((0, 255, 255))
#             else:
#                 color = emotion_probability * np.asarray((0, 255, 0))

#             color = color.astype(int).tolist()

#             draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#             draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode, color, 0, -45, 1, 1)

#             # Update shared emotion data
#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#             # Send emotion data to ESP32 via Flask server
#             try:
#                 response = requests.post(ESP32_IP, json=emotion_data)
#                 print(f'Sent data to ESP32: {response.status_code}')
#             except Exception as e:
#                 print(f'Error sending data to ESP32: {e}')

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         # cv2.imshow('window_frame', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# # Define a route to receive emotion data from ESP32
# @app.route('/emotion', methods=['POST'])
# def receive_emotion_data():
#     print("Received POST request at /emotion endpoint")
#     if request.method == 'POST':
#         try:
#             data = request.json  # Get JSON data from the request
#             emotion = data['emotion']
#             probability = data['probability']
#             sensor_values = data.get('sensor_values', None)  # Optional sensor values
#             # Process the received data as needed
#             print(f"Received emotion: {emotion}, Probability: {probability}")
#             if sensor_values:
#                 print(f"Received sensor values: {sensor_values}")
#             # Perform actions based on the received data
#             # For example, trigger some functions or update a database
#             # Respond with a success message
#             return jsonify({'message': 'Data received successfully'}), 200
#         except Exception as e:
#             print(f"Error processing request: {e}")
#             return jsonify({'error': 'Failed to process request'}), 400

# if __name__ == "__main__":
#     threading.Thread(target=detect_emotions).start()  # Start emotion detection in a separate thread
#     app.run(host='0.0.0.0', port=5000)  # Run Flask app on port 5000, allowing external connections


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import requests
# import json
# from flask import Flask, request, jsonify

# app = Flask(__name__)

# # Constants
# USE_WEBCAM = True
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# ESP32_IP = 'http://192.168.0.17/emotion'

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Video capture
# cap = cv2.VideoCapture(0)

# # Shared variable for emotions
# emotion_data = []

# def detect_emotions():
#     global emotion_data
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             if emotion_text == 'angry':
#                 color = emotion_probability * np.asarray((255, 0, 0))
#             elif emotion_text == 'sad':
#                 color = emotion_probability * np.asarray((0, 0, 255))
#             elif emotion_text == 'happy':
#                 color = emotion_probability * np.asarray((255, 255, 0))
#             elif emotion_text == 'surprise':
#                 color = emotion_probability * np.asarray((0, 255, 255))
#             else:
#                 color = emotion_probability * np.asarray((0, 255, 0))

#             color = color.astype(int).tolist()

#             draw_bounding_box(face_utils.rect_to_bb(face_coordinates), rgb_image, color)
#             draw_text(face_utils.rect_to_bb(face_coordinates), rgb_image, emotion_mode, color, 0, -45, 1, 1)

#             # Update shared emotion data
#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#             # Send emotion data to ESP32 via Flask server
#             try:
#                 print("Sending data to ESP32...")
#                 response = requests.post(ESP32_IP, json=emotion_data)
#                 print(f'Sent data to ESP32: {response.status_code}')
#             except Exception as e:
#                 print(f'Error sending data to ESP32: {e}')

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         # cv2.imshow('window_frame', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# # Define a route to receive emotion data from ESP32
# @app.route('/emotion', methods=['POST'])
# def receive_emotion_data():
#     print("Received POST request at /emotion endpoint")
#     if request.method == 'POST':
#         try:
#             data = request.json  # Get JSON data from the request
#             emotion = data['emotion']
#             probability = data['probability']
#             sensor_values = data.get('sensor_values', None)  # Optional sensor values
#             # Process the received data as needed
#             print(f"Received emotion: {emotion}, Probability: {probability}")
#             if sensor_values:
#                 print(f"Received sensor values: {sensor_values}")
#             # Perform actions based on the received data
#             # For example, trigger some functions or update a database
#             # Respond with a success message
#             return jsonify({'message': 'Data received successfully'}), 200
#         except Exception as e:
#             print(f"Error processing request: {e}")
#             return jsonify({'error': 'Failed to process request'}), 400

# if __name__ == "__main__":
#     threading.Thread(target=detect_emotions).start()  # Start emotion detection in a separate thread
#     app.run(host='0.0.0.0', port=5000)  # Run Flask app on port 5000, allowing external connections


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import asyncio
# import websockets 
# import json

# # Constants
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# client_address = "192.168.0.17"

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Shared variable for emotions
# emotion_data = {'emotion': "", 'probability': 0.0}

# async def handle_emotion(websocket, path):
#     global emotion_data
#     try:
#         async for message in websocket:
#             data = json.loads(message)
#             emotion_data['emotion'] = data['emotion']
#             emotion_data['probability'] = data['probability']
#             print(f"Received emotion: {emotion_data['emotion']}, Probability: {emotion_data['probability']}")
#     except Exception as e:
#         print(f"Error: {e}")

# def detect_emotions():
#     global emotion_data
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#         # Control pumps based on the received emotion
#         control_pumps(emotion_data['emotion'], emotion_data['probability'])

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         # cv2.imshow('Emotion Detection', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# def control_pumps(emotion, probability):
#     # Control pumps based on the received emotion and probability
#     if emotion == 'angry':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = 0
#     elif emotion == 'sad':
#         pump1_intensity = 0
#         pump2_intensity = int(probability * 255)
#     elif emotion == 'happy':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255)
#     elif emotion == 'surprise':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255 / 2)
#     else:
#         pump1_intensity = 0
#         pump2_intensity = 0

#     # Apply pump intensities
#     # Replace these with your actual pump control logic
#     print(f"Pump 1 Intensity: {pump1_intensity}")
#     print(f"Pump 2 Intensity: {pump2_intensity}")

# if __name__ == "__main__":
#     threading.Thread(target=detect_emotions).start()
#     start_server = websockets.serve(handle_emotion, '0.0.0.0', 5000)
#     asyncio.get_event_loop().run_until_complete(start_server)
#     asyncio.get_event_loop().run_forever()


# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import asyncio
# import websockets 
# import json

# # Constants
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# client_address = "192.168.0.17"

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Shared variable for emotions
# emotion_data = {'emotion': "", 'probability': 0.0}

# # Dictionary mapping emotions to sentences
# emotion_sentences = {
#     'angry': "You seem angry.",
#     'sad': "You seem sad.",
#     'happy': "You seem happy.",
#     'surprise': "You seem surprised.",
#     'neutral': "You seem neutral."
# }

# async def handle_emotion(websocket, path):
#     global emotion_data
#     try:
#         async for message in websocket:
#             data = json.loads(message)
#             emotion_data['emotion'] = data['emotion']
#             emotion_data['probability'] = data['probability']
#             print(f"Received emotion: {emotion_data['emotion']}, Probability: {emotion_data['probability']}")
#     except Exception as e:
#         print(f"Error: {e}")

# def detect_emotions():
#     global emotion_data
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#         # Control pumps based on the received emotion
#         control_pumps(emotion_data['emotion'], emotion_data['probability'])

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         # cv2.imshow('Emotion Detection', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# def control_pumps(emotion, probability):
#     # Print the corresponding sentence based on the emotion
#     print(emotion_sentences.get(emotion, "Unknown emotion."))

#     # Control pumps based on the received emotion and probability
#     if emotion == 'angry':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = 0
#     elif emotion == 'sad':
#         pump1_intensity = 0
#         pump2_intensity = int(probability * 255)
#     elif emotion == 'happy':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255)
#     elif emotion == 'surprise':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255 / 2)
#     else:
#         pump1_intensity = 0
#         pump2_intensity = 0

#     # Apply pump intensities
#     # Replace these with your actual pump control logic
#     print(f"Pump 1 Intensity: {pump1_intensity}")
#     print(f"Pump 2 Intensity: {pump2_intensity}")

# if __name__ == "__main__":
#     threading.Thread(target=detect_emotions).start()
#     start_server = websockets.serve(handle_emotion, '0.0.0.0', 5000)
#     asyncio.get_event_loop().run_until_complete(start_server)
#     asyncio.get_event_loop().run_forever()

# SENDING THE TEXT PROPERLY 

# import cv2
# import numpy as np
# import dlib
# from imutils import face_utils
# from keras.models import load_model
# from statistics import mode
# from utils.datasets import get_labels
# from utils.inference import draw_text, draw_bounding_box, apply_offsets, preprocess_input
# import threading
# import asyncio
# import websockets 
# import json

# # Constants
# emotion_model_path = 'models/emotion_model.hdf5'
# emotion_labels = get_labels('fer2013')
# frame_window = 10
# emotion_offsets = (20, 40)
# # client_address = "192.168.0.17"
# client_address = "172.16.22.121"

# # Load models
# detector = dlib.get_frontal_face_detector()
# emotion_classifier = load_model(emotion_model_path)
# emotion_target_size = emotion_classifier.input_shape[1:3]
# emotion_window = []

# # Shared variable for emotions
# emotion_data = {'emotion': "", 'probability': 0.0}

# async def handle_emotion(websocket, path):
#     global emotion_data
#     try:
#         async for message in websocket:
#             data = json.loads(message)
#             emotion_data['emotion'] = data['emotion']
#             emotion_data['probability'] = data['probability']
#             print(f"Received emotion: {emotion_data['emotion']}, Probability: {emotion_data['probability']}")
#     except Exception as e:
#         print(f"Error: {e}")

# def detect_emotions():
#     global emotion_data
#     cap = cv2.VideoCapture(0)
#     while cap.isOpened():
#         ret, bgr_image = cap.read()
#         if not ret:
#             continue
#         gray_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2GRAY)
#         rgb_image = cv2.cvtColor(bgr_image, cv2.COLOR_BGR2RGB)

#         faces = detector(rgb_image)
#         for face_coordinates in faces:
#             x1, x2, y1, y2 = apply_offsets(face_utils.rect_to_bb(face_coordinates), emotion_offsets)
#             gray_face = gray_image[y1:y2, x1:x2]
#             try:
#                 gray_face = cv2.resize(gray_face, (emotion_target_size))
#             except:
#                 continue

#             gray_face = preprocess_input(gray_face, True)
#             gray_face = np.expand_dims(gray_face, 0)
#             gray_face = np.expand_dims(gray_face, -1)
#             emotion_prediction = emotion_classifier.predict(gray_face)
#             emotion_probability = np.max(emotion_prediction)
#             emotion_label_arg = np.argmax(emotion_prediction)
#             emotion_text = emotion_labels[emotion_label_arg]
#             emotion_window.append(emotion_text)

#             if len(emotion_window) > frame_window:
#                 emotion_window.pop(0)
#             try:
#                 emotion_mode = mode(emotion_window)
#             except:
#                 continue

#             emotion_data = {'emotion': emotion_text, 'probability': float(emotion_probability)}

#         # Control pumps based on the received emotion
#         control_pumps(emotion_data['emotion'], emotion_data['probability'])

#         bgr_image = cv2.cvtColor(rgb_image, cv2.COLOR_RGB2BGR)
#         # cv2.imshow('Emotion Detection', bgr_image)
#         if cv2.waitKey(1) & 0xFF == ord('q'):
#             break

#     cap.release()
#     cv2.destroyAllWindows()

# def control_pumps(emotion, probability):
#     # Control pumps based on the received emotion and probability
#     if emotion == 'angry':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = 0
#         sentence = "Is it really worth it? Let it go."
#     elif emotion == 'sad':
#         pump1_intensity = 0
#         pump2_intensity = int(probability * 255)
#         sentence = "Its just a moment! Feel better soon."
#     elif emotion == 'happy':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255)
#         sentence = "Yay! Lets party."
#     elif emotion == 'surprise':
#         pump1_intensity = int(probability * 255)
#         pump2_intensity = int(probability * 255 / 2)
#         sentence = "Wow."
#     else:
#         pump1_intensity = 0
#         pump2_intensity = 0
#         sentence = "What is feeling?"

#     # Apply pump intensities
#     # Replace these with your actual pump control logic
#     print(f"Pump 1 Intensity: {pump1_intensity}")
#     print(f"Pump 2 Intensity: {pump2_intensity}")

#     # Write the emotion, probability, and sentence to a text file
#     with open('emotion_sentences.txt', 'a') as file:
#         file.write(f"{sentence}\n")

# if __name__ == "__main__":
#     threading.Thread(target=detect_emotions).start()
#     start_server = websockets.serve(handle_emotion, '0.0.0.0', 5000)
#     asyncio.get_event_loop().run_until_complete(start_server)
#     asyncio.get_event_loop().run_forever()
